@@ -5,15 +5,19 @@
 :copyright: 2007-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
+from __future__ import with_statement
+
 __docformat__ = "restructuredtext en"
 
 import sys
+from os.path import join
 
 from logilab.common.modutils import LazyObject
 from logilab.mtconverter import guess_mimetype_and_encoding
 from yams import ValidationError
 
 from cubicweb import Binary, QueryError, typed_eid
+from cubicweb.server.session import security_enabled
 from cubes.vcsfile import queries
 
 _VCSTYPES = {'subversion': LazyObject('cubes.vcsfile.reposvn', 'SVNRepository'),
@@ -38,28 +42,32 @@ class VCSException(Exception):
         return ValidationError(self.repoeid, {self.attr: msg})
 
 
-
 def repository_handler(repoent):
     """get repository handler for the given repository entity"""
-    try:
-        repohdlr = _REPOHDLRS[repoent.eid]
-        repohdlr.encoding = repoent.encoding # in case it changed
-        return repohdlr
-    except KeyError:
-        pass
-    try:
-        path = repoent.path
-        if not path:
-            path = repoent.local_cache
-        repohdlr = _VCSTYPES[repoent.type](repoent.eid, path, repoent.encoding)
-    except KeyError:
-        msg = repoent._cw._('%s is not a known repository type')
-        raise ValidationError(repoent.eid, {'type': msg % repoent.type})
-    except ImportError:
-        msg = repoent._cw._('missing python bindings to support %s repositories')
-        raise ValidationError(repoent.eid, {'type': msg % repoent.type})
-    _REPOHDLRS[repoent.eid] = repohdlr
-    return repohdlr
+    # deactivate read security first, potentially needed to get access to
+    # path/local_cache
+    session = repoent._cw
+    with security_enabled(session, read=False):
+        try:
+            repohdlr = _REPOHDLRS[repoent.eid]
+            repohdlr.encoding = repoent.encoding # in case it changed
+            return repohdlr
+        except KeyError:
+            try:
+                path = repoent.path
+                if not path:
+                    path = join(session.vreg.config['local-repo-cache-root'],
+                                repoent.local_cache)
+                repohdlr = _VCSTYPES[repoent.type](repoent.eid, path,
+                                                   repoent.encoding)
+            except KeyError:
+                msg = session._('%s is not a known repository type')
+                raise ValidationError(repoent.eid, {'type': msg % repoent.type})
+            except ImportError:
+                msg = session._('missing python bindings to support %s repositories')
+                raise ValidationError(repoent.eid, {'type': msg % repoent.type})
+            _REPOHDLRS[repoent.eid] = repohdlr
+            return repohdlr
 
 
 def import_content(session, repo, commitevery=10):
@@ -68,8 +76,9 @@ def import_content(session, repo, commitevery=10):
     `repo` is the cubicweb repository (not a vcs repository)
     """
     for repoentity in session.execute(
-        'Any X,T,P,SP,E WHERE X is Repository, '
-        'X type T, X path P, X subpath SP, X encoding E').entities():
+        'Any X,T,U,P,L,SP,E WHERE X is Repository, '
+        'X type T, X source_url U, X path P, X local_cache L, '
+        'X subpath SP, X encoding E').entities():
         # give a change to threads waiting for a pool
         session.set_pool()
         try:
