@@ -40,7 +40,7 @@ class Repository(AnyEntity):
     __regid__ ='Repository'
 
     __permissions__ = ('write',)
-    fetch_attrs, fetch_order = fetch_config(['title', 'path', 'subpath', 'type'])
+    fetch_attrs, fetch_order = fetch_config(['source_url', 'title', 'path', 'subpath', 'type'])
     rest_attr = 'eid' # see #XXX, using path cause pb w/ apache redirection
 
     def dc_title(self):
