@@ -24,6 +24,8 @@ def repo_cache_dir(config):
     directory = config['local-repo-cache-root']
     if not osp.isabs(directory):
         directory = osp.join(config.appdatahome, directory)
+        if not config.repairing: # don't change value during migration
+            config['local-repo-cache-root'] = directory
     if not osp.exists(directory):
         try:
             os.makedirs(directory)
@@ -34,12 +36,40 @@ def repo_cache_dir(config):
             raise
     return directory
 
-def url_to_relative_local_path(url):
+def url_to_relative_path(url):
     scheme, url = url.split('://', 1)
     if scheme == 'file':
-        return url.rsplit('/', 1)[-1]
+        return url.rstrip('/').rsplit('/', 1)[-1]
     else:
-        return url.rsplit('/', 1)[1]
+        return url.rstrip('/').rsplit('/', 1)[1]
+
+def set_local_cache(vcsrepo):
+    cachedir = osp.join(repo_cache_dir(vcsrepo._cw.vreg.config),
+                        unicode(vcsrepo.eid))
+    if not osp.exists(cachedir):
+        try:
+            os.makedirs(cachedir)
+        except OSError:
+            vcsrepo.exception('cant create repo cache directory %s', cachedir)
+            return
+    try:
+        vcsrepo['local_cache'] = osp.join(
+            unicode(vcsrepo.eid), url_to_relative_path(vcsrepo.source_url))
+    except (IndexError, ValueError):
+        # do nothing, validation error should be raised by another hook
+        pass
+
+def clone_to_local_cache(vcsrepo):
+    handler = bridge.repository_handler(vcsrepo)
+    url = vcsrepo.source_url
+    try:
+        handler.pull_or_clone_cache(url)
+    except Exception:
+        handler.exception('while trying to clone repo %s', url)
+        msg = vcsrepo._cw._('can not clone the repo from %s, '
+                            'please check the source url')
+        raise ValidationError(vcsrepo.eid, {'source_url': msg % url})
+
 
 def missing_relation_error(entity, rtype):
     # use __ since msgid recorded in cw, we don't want to translate it in
@@ -52,15 +82,13 @@ def missing_relation_error(entity, rtype):
     return ValidationError(entity.eid, errors)
 
 def missing_attribute_error(entity, attrs):
-    # use __ since msgid recorded in cw, we don't want to translate it in
-    # this cubex
-    __ = entity._cw._
     msg = _('at least one attribute of %s must be set on a Repository')
     errors = {}
     for attr in attrs:
         errors[attr] = msg % ', '.join(attrs)
     return ValidationError(entity.eid, errors)
 
+
 # initialization hooks #########################################################
 
 class ServerMaintenanceHook(Hook):
@@ -89,22 +117,23 @@ class ServerStartupHook(Hook):
             raise ValueError, ('directory %r is not writable '
                                '(check  option)' % (cacheroot, CACHEROOT))
 
-        def refresh_local_hgrepo_caches(repo, osp=osp, bridge=bridge, cacheroot=cacheroot):
+        def refresh_local_hgrepo_caches(repo, osp=osp, bridge=bridge,
+                                        cacheroot=cacheroot):
             session = repo.internal_session()
             try:
-                reposrset = session.execute('Any R, RT, RLC, RSU, RP WHERE R is Repository, '
-                                            'R type "mercurial", NOT R local_cache NULL, '
-                                            'R type RT, R local_cache RLC, '
-                                            'R source_url RSU, R path RP')
-                for repository in reposrset.entities():
-                    repo.info('refreshing local repository caches for %s', repository.eid)
-                    # do stuff with repositories in the cacheroot
-                    repohandler = bridge.repository_handler(repository)
-                    if not osp.exists(repository.local_cache):
-                        repo.info('cloning into local cache %s', repository.source_url)
-                        repohandler.clone_to_cache(repository)
-                    else:
-                        repohandler.pull_on_cache(repository)
+                reposrset = session.execute(
+                    'Any R, RT, RLC, RSU, RP WHERE R is Repository, '
+                    'R type "mercurial", NOT R local_cache NULL, '
+                    'R type RT, R local_cache RLC, '
+                    'R source_url RSU, R path RP')
+                for vcsrepo in reposrset.entities():
+                    try:
+                        bridge.repository_handler(vcsrepo).pull_or_clone_cache(
+                            vcsrepo.source_url)
+                    except Exception:
+                        self.exception(
+                            'error while updating local cache of repository %s',
+                            vcsrepo.eid)
                     # give other threads a chance to run
                     session.set_pool()
                 if repo.config['repository-import']:
@@ -150,9 +179,10 @@ class VFModificationDateOp(Operation):
 
 class UpdateVFModificationDateHook(Hook):
     __regid__ = 'vcsfile.update_vf_mdate'
-    events = ('after_add_entity',)
     __select__ = Hook.__select__ & is_instance('DeletedVersionContent',
-                                              'VersionContent')
+                                               'VersionContent')
+    events = ('after_add_entity',)
+
     def __call__(self):
         set_operation(self._cw, 'vcsfile.modifiedvfs', self.entity,
                       VFModificationDateOp)
@@ -160,8 +190,8 @@ class UpdateVFModificationDateHook(Hook):
 
 class AddRevisionHook(Hook):
     __regid__ = 'vcsfile.add_revision_hook'
-    events = ('before_add_entity',)
     __select__ = Hook.__select__ & is_instance('Revision')
+    events = ('before_add_entity',)
 
     def __call__(self):
         entity = self.entity
@@ -197,8 +227,8 @@ class AddRevisionHook(Hook):
 
 class AddVersionContentHook(Hook):
     __regid__ = 'vcsfile.add_version_content_hook'
-    events = ('before_add_entity',)
     __select__ = Hook.__select__ & is_instance('VersionContent')
+    events = ('before_add_entity',)
 
     def __call__(self):
         session = self._cw
@@ -217,8 +247,8 @@ class AddVersionContentHook(Hook):
 
 class AddDeletedVersionContentHook(Hook):
     __regid__ = 'vcsfile.add_deleted_version_content_hook'
-    events = ('before_add_entity',)
     __select__ = Hook.__select__ & is_instance('DeletedVersionContent')
+    events = ('before_add_entity',)
 
     def __call__(self):
         # skip further processing if the revision is being imported from the
@@ -233,8 +263,8 @@ class AddDeletedVersionContentHook(Hook):
 
 class AddVersionedFileHook(Hook):
     __regid__ = 'vcsfile.add_versioned_file_hook'
-    events = ('before_add_entity',)
     __select__ = Hook.__select__ & is_instance('VersionedFile')
+    events = ('before_add_entity',)
 
     def __call__(self):
         # skip further processing if the revision is being imported from the
@@ -252,23 +282,12 @@ class SetRepoCacheBeforeAdd(Hook):
     __select__ = (Hook.__select__ & is_instance('Repository'))
     events = ('before_add_entity',)
     order = 2
+
     def __call__(self):
         repo = self.entity
         if repo.type == 'mercurial' and repo.source_url and not repo.path:
-            cachedir = osp.join(repo_cache_dir(self._cw.vreg.config),
-                                unicode(repo.eid))
-            if not osp.exists(cachedir):
-                try:
-                    os.makedirs(cachedir)
-                except OSError:
-                    self.exception('cant create repo cache directory %s', cachedir)
-                    return
-            try:
-                repo['local_cache'] = osp.join(
-                    cachedir, url_to_relative_local_path(repo.source_url))
-            except (IndexError, ValueError):
-                # do nothing, validation error should be raised by another hook
-                pass
+            set_local_cache(repo)
+
 
 class CloneRepoAfterAdd(Hook):
     """clone just after repo creation, to allow operation which needs local
@@ -280,23 +299,32 @@ class CloneRepoAfterAdd(Hook):
     order = 2
 
     def __call__(self):
-        repository = self.entity
-        if not repository.local_cache:
+        vcsrepo = self.entity
+        if not vcsrepo.local_cache:
             return
-        self.info('cloning into local cache %s', repository.source_url)
-        repohandler = bridge.repository_handler(repository)
-        try:
-            # may already exist during test
-            if not osp.exists(repository.local_cache):
-                repohandler.clone_to_cache(repository)
-            else:
-                repohandler.pull_on_cache(repository)
-        except:
-            self.exception('while trying to clone repo %s',
-                           repository.source_url)
-            msg = self._cw._('cannot clone the repo from %s, please check the source url')
-            msg %= repository.source_url
-            raise ValidationError(repository.eid, {'source_url': msg})
+        # XXX rmdir on rollback
+        clone_to_local_cache(vcsrepo)
+
+
+class UpdateRepositoryHook(Hook):
+    """add repository eid to vcs bridge cache"""
+    __regid__ = 'vcsfile.update_repository_hook'
+    __select__ = Hook.__select__ & is_instance('Repository')
+    events = ('before_update_entity', )
+
+    def __call__(self):
+        vcsrepo = self.entity
+        if 'type' in vcsrepo.edited_attributes:
+            msg = self._cw._('updating type attribute of a repository isn\'t '
+                             'supported. Delete it and add a new one.')
+            raise ValidationError(vcsrepo.eid, {'type': msg})
+        if 'path' in vcsrepo.edited_attributes or 'local_cache' in vcsrepo.edited_attributes:
+            bridge._REPOHDLRS.pop(vcsrepo.eid, None)
+        if (vcsrepo.type == 'mercurial' and vcsrepo.source_url
+            and not vcsrepo.path and not vcsrepo.local_cache):
+            # XXX rmdir on rollback
+            set_local_cache(vcsrepo)
+            clone_to_local_cache(vcsrepo)
 
 
 class DeleteRepositoryHook(Hook):
@@ -307,15 +335,23 @@ class DeleteRepositoryHook(Hook):
     events = ('before_delete_entity',)
 
     def __call__(self):
-        set_operation(self._cw, 'delete_repo_cleanup', self.entity.local_cache, DeleteRepositoryOp)
+        if self.entity.local_cache:
+            cachedir = osp.dirname(osp.join(
+                self._cw.vreg.config['local-repo-cache-root'],
+                self.entity.local_cache)).encode('ascii')
+            set_operation(self._cw, 'vcsfile.deletedirs', cachedir,
+                          DeleteDirsOp)
 
-class DeleteRepositoryOp(Operation):
+class DeleteDirsOp(Operation):
 
     def postcommit_event(self):
-        session = self.session
-        for cache in session.transaction_data.pop('delete_repo_cleanup'):
-            self.info('deleting repository cache at %s', cache)
-            shutil.rmtree(cache.encode('ascii'))
+        for cachedir in self.session.transaction_data.pop('vcsfile.deletedirs'):
+            try:
+                shutil.rmtree(cachedir)
+                self.info('deleted repository cache at %s', cachedir)
+            except:
+                self.exception('cant delete repository cache at %s', cachedir)
+
 
 # safety belts #################################################################
 
@@ -358,9 +394,9 @@ class CheckRevisionOp(Operation):
 
 class CheckImmutableAttributeHook(Hook):
     __regid__ = 'vcsfile.check_immutable_attribute_hook'
+    __select__ = Hook.__select__ & is_instance(
+        'Revision', 'DeletedVersionContent', 'VersionContent')
     events = ('before_update_entity',)
-    __select__ = Hook.__select__ & is_instance('Revision', 'DeletedVersionContent',
-                                              'VersionContent')
 
     def __call__(self):
         for attr in self.entity.keys():
@@ -370,29 +406,6 @@ class CheckImmutableAttributeHook(Hook):
                 raise QueryError('%s attribute is not editable' % attr)
 
 
-class UpdateRepositoryHook(Hook):
-    """add repository eid to vcs bridge cache"""
-    __regid__ = 'vcsfile.update_repository_hook'
-    events = ('before_update_entity', )
-    __select__ = Hook.__select__ & is_instance('Repository')
-    category = 'vcsfile.repo_immutable_attrs'
-
-    def __call__(self):
-        # XXX check value actually changed
-        try:
-            edited = self.entity.edited_attributes
-        except AttributeError:
-            edited = self.entity
-        errors = []
-        for attr in ('path', 'type', 'source_url'):
-            if attr in edited:
-                msg = self._cw._('updating %s attribute of a repository isn\'t '
-                                 'supported. Delete it and add a new one.') %  attr
-                errors.append((attr, msg))
-        if errors:
-            raise ValidationError(self.entity.eid, dict(errors))
-
-
 class AddUpdateRepositoryHook(Hook):
     __regid__ = 'vcsfile.add_update_repository_hook'
     __select__ = Hook.__select__ & is_instance('Repository')
@@ -412,7 +425,7 @@ class AddUpdateRepositoryHook(Hook):
             raise missing_attribute_error(entity, self.required_attrs)
         if entity.source_url and not entity.path and not entity.local_cache:
             try:
-                url_to_relative_local_path(entity.source_url)
+                url_to_relative_path(entity.source_url)
             except:
                 msg = self._cw._('url must be of the form protocol://path/to/stuff')
                 raise ValidationError(entity.eid, {'source_url': msg})
