@@ -6,8 +6,8 @@
 """
 __docformat__ = "restructuredtext en"
 
-from os import umask
-from os.path import split, exists
+from os import umask, makedirs
+from os.path import split, exists, join, dirname
 from datetime import datetime
 
 from mercurial import hg, ui, revlog, context, util, commands as hgcmd
@@ -21,6 +21,9 @@ _DELETED = object()
 
 class vcsfile_ui(ui.ui):
     """ redirect output to cubicweb log """
+    def __init__(self, loggable):
+        ui.ui.__init__(self)
+        self.cw_loggable = loggable
 
     def copy(self):
         return self
@@ -38,7 +41,7 @@ class vcsfile_ui(ui.ui):
 
     def write_err(self, *args, **kwargs):
         for a in args:
-            self.cw_loggable.debug(str(a).strip())
+            self.cw_loggable.error(str(a).strip())
 
 
 def bw_nb_revisions(changelog):
@@ -54,26 +57,25 @@ def bw_nb_revisions(changelog):
 class HGRepository(repo.VCSRepository):
     type = 'hg'
 
-    def clone_to_cache(self, repoent):
-        vui = vcsfile_ui()
-        vui.cw_loggable = repoent
-        hgcmd.clone(vui,
-                    repoent.source_url.encode('ascii'),
-                    dest=repoent.local_cache.encode('ascii'),
-                    stream='compressed',
-                    noupdate=True)
-
-    def pull_on_cache(self, repoent):
-        lui = vcsfile_ui()
-        lui.cw_loggable = repoent
-        hgrepo = hg.repository(lui, repoent.local_cache)
-        hgcmd.pull(lui, hgrepo, source=repoent.source_url.encode('ascii'))
+    def pull_or_clone_cache(self, url):
+        if not exists(self.path):
+            pdir = dirname(self.path)
+            if not exists(pdir):
+                makedirs(pdir)
+            self.info('cloning %s into %s', url, self.path)
+            hgcmd.clone(vcsfile_ui(self), str(url),
+                        dest=self.path, stream='compressed', noupdate=True)
+        else:
+            self.info('pulling from %s into %s', url, self.path)
+            vui = vcsfile_ui(self)
+            hgcmd.pull(vui, hg.repository(vui, self.path), source=str(url))
 
     def import_content(self, repoentity, commitevery=10):
         """import content from the mercurial repository"""
         assert self.eid == repoentity.eid
-        if not exists(repoentity.path or repoentity.local_cache):
-            self.info('repository %s not accessible', repoentity.dc_title())
+        if not exists(self.path):
+            self.info('repository %s not accessible in %s',
+                      repoentity.dc_title(), self.path)
             return
         try:
             repo = self.hgrepo()
@@ -85,25 +87,25 @@ class HGRepository(repo.VCSRepository):
         actualrev = bw_nb_revisions(repo.changelog) - 1
         session = repoentity._cw
         latestrev = repoentity.latest_known_revision()
-        repopath = repoentity.path or repoentity.local_cache
         if actualrev < latestrev:
             # striped repo
             # very minimal support, this won't work if local number of
             # revisions has changed due to the strip
-            self.warning('%s: strip detected ? latest known revision is %s, latest '
-                         'repository revision is %s', repopath, latestrev, actualrev)
+            self.warning('%s: strip detected ? latest known revision is %s, '
+                         'latest repository revision is %s', self.path,
+                         latestrev, actualrev)
             session.execute('DELETE Revision X WHERE X revision > %(rev)s',
                             {'rev': actualrev})
             return
         self.info('hg repo %s: known revisions up to %s, repo revision %s',
-                  repopath, latestrev, actualrev)
+                  self.path, latestrev, actualrev)
         for i in xrange(latestrev + 1, actualrev + 1):
             self.info('importing revision %s', i)
             try:
                 self.import_revision(repoentity, repo, i)
             except:
                 self.critical('error while importing revision %s of %s',
-                              i, repopath, exc_info=True)
+                              i, self.path, exc_info=True)
                 raise
             if not i % commitevery:
                 # give a change to threads waiting for a pool by freeing it then
