@@ -16,7 +16,7 @@ class HooksTC(VCSFileTC):
 
 
 class AtRevisionHooksTC(VCSFileTC):
-    repo_path = u'testrepohg_branches'
+    _repo_path = u'testrepohg_branches'
     repo_type = u'mercurial'
 
     def test_at_revision_is_set(self):
