@@ -13,12 +13,12 @@ from utils import VCSFileTC
 REPOBASEPATH = abspath(split(__file__)[0])
 
 class _WriteTC(VCSFileTC):
-    repo_path = u'testrepocopy'
+    _repo_path = u'testrepocopy'
     def setUp(self):
         try:
             copytree(str(self.orig_repo_path), str(self.repo_path))
             VCSFileTC.setUp(self)
-            self.copath = join(REPOBASEPATH, self.repo_path)
+            self.copath = join(REPOBASEPATH, self._repo_path)
         except:
             rmtree(str(self.repo_path))
             raise
@@ -26,7 +26,7 @@ class _WriteTC(VCSFileTC):
 
     def tearDown(self):
         VCSFileTC.tearDown(self)
-        rmtree(str(self.repo_path))
+        rmtree(str(self._repo_path))
 
     def _test_new_revision(self, entity, vf, rev, revision):
         self.assertEquals(entity.content_for[0].eid, vf.eid)
@@ -53,7 +53,11 @@ class _WriteTC(VCSFileTC):
         vf, rev, vc = self._new_toto_revision()
         self.commit()
         self._test_new_revision(vc, vf, rev, 8 + self.repo_rev_offset)
-        bridge.import_content(self.repo)
+        try:
+            session = self.repo.internal_session()
+            bridge.import_content(session, self.repo)
+        finally:
+            session.close()
         self.assertEquals(len(self.execute('Any X WHERE X revision %(r)s', {'r': rev.revision})), 1)
 
     def test_new_revision_security(self):
@@ -322,7 +326,11 @@ class HGSourceWriteTC(_WriteTC):
         self.assertEquals(self.vcsrepo.latest_known_revision(), 6)
         vcrset = self.execute('Any X WHERE X from_revision R, R revision 6')
         system('cd %s; hg --config extensions.hgext.mq= strip 6 2>/dev/null' % self.copath)
-        bridge.import_content(self.repo)
+        try:
+            session = self.repo.internal_session()
+            bridge.import_content(session, self.repo)
+        finally:
+            session.close()
         self.commit()
         self.assertEquals(self.vcsrepo.latest_known_revision(), 5)
         for eid, in vcrset:
@@ -334,7 +342,11 @@ class HGSourceWriteTC(_WriteTC):
         system('cd %s; hg up default >/dev/null 2>&1' % hgdir)
         system('cd %s; hg branch newbranch >/dev/null 2>&1' % hgdir)
         system('cd %s; hg tag newbranch -m "opening newbranch"' % hgdir)
-        bridge.import_content(self.repo)
+        try:
+            session = self.repo.internal_session()
+            bridge.import_content(session, self.repo)
+        finally:
+            session.close()
         self.commit()
         hgtags = self.execute('VersionContent V WHERE V content_for VF,  '
                               'VF name ".hgtags"').get_entity(0,0)
