@@ -7,7 +7,7 @@
 __docformat__ = "restructuredtext en"
 _ = unicode
 
-from cubicweb.selectors import (one_line_rset, implements, rql_condition,
+from cubicweb.selectors import (one_line_rset, is_instance, rql_condition,
                                 but_etype, score_entity)
 from cubicweb.web.action import Action
 from cubicweb.web.views.actions import CopyAction
@@ -16,7 +16,7 @@ from cubes.vcsfile.entities import rql_revision_content
 
 
 class VCAction(Action):
-    __select__ = one_line_rset() & implements('VersionContent', 'DeletedVersionContent')
+    __select__ = one_line_rset() & is_instance('VersionContent', 'DeletedVersionContent')
     category = 'mainactions'
 
 
@@ -83,7 +83,7 @@ class VCRevisionAllFilesAction(VCAction):
 
 
 class VFAction(Action):
-    __select__ = one_line_rset() & implements('VersionedFile')
+    __select__ = one_line_rset() & is_instance('VersionedFile')
     category = 'mainactions'
 
 
@@ -112,7 +112,7 @@ class VFHEADRevisionAction(VFAction):
 
 
 # deactivate copy action for entities coming from the svn source
-CopyAction.__select__ = CopyAction.__select__ & ~implements(
+CopyAction.__select__ = CopyAction.__select__ & ~is_instance(
     'Revision', 'VersionedFile', 'VersionContent', 'DeletedVersionContent')
 
 
