@@ -27,9 +27,10 @@ _afs = uicfg.autoform_section
 _aff = uicfg.autoform_field
 _affk = uicfg.autoform_field_kwargs
 _rctl = uicfg.reledit_ctrl
+_pvdc = uicfg.primaryview_display_ctrl
 
 for attr in ('path', 'subpath', 'type', 'source_url', 'encoding'):
-    _rctl.tag_attribute(('Repository', attr), {'noedit': True})
+    _pvdc.tag_attribute(('Repository', attr), {'vid': 'attribute'})
 _rctl.tag_attribute(('Repository', 'title'), {'reload': True})
 _affk.tag_attribute(('Repository', 'source_url'), {'widget': fw.TextInput})
 _affk.tag_attribute(('Repository', 'path'), {'widget': fw.TextInput})
