@@ -12,14 +12,11 @@ from logilab.mtconverter import TransformError, xml_escape
 from cubicweb import tags
 from cubicweb.mttransforms import ENGINE
 from cubicweb.selectors import is_instance
-from cubicweb.entities import adapters
 from cubicweb.view import EntityView
 from cubicweb.web import uicfg
 from cubicweb.web.views import (ibreadcrumbs, idownloadable,
                                 baseviews, primary, tabs, navigation)
 
-from cubicweb.web import uicfg
-
 # primary view tweaks
 _pvs = uicfg.primaryview_section
 _pvdc = uicfg.primaryview_display_ctrl
@@ -28,6 +25,7 @@ _abaa = uicfg.actionbox_appearsin_addmenu
 # internal purpose relation
 _pvs.tag_subject_of(('*', 'at_revision', '*'), 'hidden')
 _pvs.tag_object_of(('*', 'at_revision', '*'), 'hidden')
+_pvs.tag_attribute(('Repository', 'local_cache'), 'hidden')
 
 _pvs.tag_attribute(('VersionedFile', 'name'), 'hidden') # in title
 _pvs.tag_attribute(('VersionedFile', 'directory'), 'hidden') # in title
@@ -48,6 +46,8 @@ _pvdc.tag_subject_of(('*', 'from_revision', '*'), {'vid': 'incontext'})
 _pvdc.tag_subject_of(('*', 'from_repository', '*'), {'vid': 'incontext'})
 _pvdc.tag_subject_of(('*', 'content_for', '*'), {'vid': 'incontext'})
 
+_pvdc.tag_attribute(('Repository', 'source_url'), {'vid': 'urlattr'})
+
 # we don't want automatic addrelated action for the following relations...
 for rtype in ('from_repository', 'from_revision', 'content_for',
               'parent_revision'):
@@ -317,21 +317,3 @@ class DVCIBreadCrumbsAdapter(ibreadcrumbs.IBreadCrumbsAdapter):
 
     def parent(self): # for breadcrumbs
         return self.entity.file
-
-
-# IDownloadable ################################################################
-
-class VersionContentIDownloadableAdapter(adapters.IDownloadableAdapter):
-    __select__ = is_instance('VersionContent')
-
-    def download_url(self, **kwargs):
-        return self.entity.absolute_url(vid='download', **kwargs)
-    def download_content_type(self):
-        return self.entity.data_format
-    def download_encoding(self):
-        return self.entity.data_encoding
-    def download_file_name(self):
-        return self.entity.file.name
-    def download_data(self):
-        return self.entity.data.getvalue()
-
